/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.asgteach.bpmonitor.views;

import com.asgteach.bpmonitor.model.BPData;
import com.asgteach.bpmonitor.model.Model;
import com.asgteach.bpmonitor.service.Service;
import com.gluonhq.charm.glisten.animation.BounceInRightTransition;
import com.gluonhq.charm.glisten.application.MobileApplication;
import static com.gluonhq.charm.glisten.application.MobileApplication.HOME_VIEW;
import com.gluonhq.charm.glisten.control.AppBar;
import com.gluonhq.charm.glisten.control.TextField;
import com.gluonhq.charm.glisten.mvc.View;
import java.time.LocalDateTime;
import javafx.beans.binding.Bindings;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.DatePicker;
import javax.inject.Inject;
import javafx.scene.control.Spinner;
import javafx.scene.control.SpinnerValueFactory;
import com.gluonhq.charm.glisten.control.Alert;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.collections.FXCollections;

/**
 *
 * @author gail
 */
public class EditPresenter {

    @Inject
    private Model model;
    @Inject
    private Service service;

    @FXML
    private View edit;
    @FXML
    private DatePicker datePicker;
    @FXML
    private ComboBox<String> ampmControl;
    @FXML
    private Spinner<Integer> hourSpinner;
    @FXML
    private Spinner<Integer> minuteSpinner;
    @FXML
    private SpinnerValueFactory.IntegerSpinnerValueFactory hourFactory;
    @FXML
    private SpinnerValueFactory.IntegerSpinnerValueFactory minuteFactory;
    @FXML
    private TextField systolicField;
    @FXML
    private TextField diastolicField;
    @FXML
    private TextField pulseField;
    @FXML
    private Button cancel;
    @FXML
    private Button submit;

    private final BooleanProperty editMode = new SimpleBooleanProperty(true);

    public void initialize() {
        edit.setShowTransitionFactory(BounceInRightTransition::new);
        edit.showingProperty().addListener((obs, oldValue, newValue) -> {
            if (newValue) {

                submit.disableProperty().unbind();
                datePicker.getParent().requestFocus();

                BPData activeBPData = model.activeBPData().get();
                if (activeBPData != null) {

                    // Populate the input fields with the information
                    // and disable the date/time controls
                    submit.setText("Update");
                    datePicker.setValue(activeBPData.getDate().toLocalDate());
                    systolicField.setText(activeBPData.getSystolic().toString());
                    diastolicField.setText(activeBPData.getDiastolic().toString());
                    pulseField.setText(activeBPData.getPulse().toString());
                    setTimeControls(activeBPData.getDate());

                    submit.disableProperty().bind(Bindings.createBooleanBinding(() -> {

                        if (systolicField == null || diastolicField == null || pulseField == null) {
                            return true;
                        }

                        return systolicField.textProperty().isEqualTo(activeBPData.getSystolic().toString())
                                .and(diastolicField.textProperty().isEqualTo(activeBPData.getDiastolic().toString()))
                                .and(pulseField.textProperty().isEqualTo(activeBPData.getPulse().toString()))
                                .get();

                    }, systolicField.textProperty(),
                            diastolicField.textProperty(),
                            pulseField.textProperty()));
                    editMode.set(true);
                } else {
                    submit.setText("Add New Reading");
                    submit.disableProperty().bind(Bindings.createBooleanBinding(() -> {
                        return systolicField.textProperty().isEmpty()
                                .or(diastolicField.textProperty().isEmpty())
                                .or(pulseField.textProperty().isEmpty())
                                .get();

                    }, systolicField.textProperty(),
                            diastolicField.textProperty(),
                            pulseField.textProperty()));
                    LocalDateTime ldate = LocalDateTime.now();
                    datePicker.setValue(ldate.toLocalDate());
                    setTimeControls(ldate);
                    editMode.set(false);
                }

                AppBar appBar = MobileApplication.getInstance().getAppBar();

                appBar.setTitleText(editMode.get() ? "Edit BP Reading" : "Add BP Reading");
            }
        });
        
        // Initialize the items in the ampmControl
        ampmControl.setItems(FXCollections.observableArrayList("am", "pm"));

        submit.setOnAction(e -> {
            BPData data = editMode.get() ? model.activeBPData().get() : new BPData();
            // Verify user entered numeric data
            try {
                // These are the BP Reading values
                data.setSystolic(Integer.parseInt(systolicField.getText()));
                data.setDiastolic(Integer.parseInt(diastolicField.getText()));
                data.setPulse(Integer.parseInt(pulseField.getText()));

                int hour = getHourSpinner();

                data.setDate(LocalDateTime.of(
                        datePicker.getValue().getYear(),
                        datePicker.getValue().getMonthValue(),
                        datePicker.getValue().getDayOfMonth(), hour,
                        minuteSpinner.getValue()));
                if (!editMode.get()) {
                    service.addReading(data);
                }

                close();

            } catch (NumberFormatException ex) {
                // Conversion error with one or more of the
                // numeric data entered in the TextField controls.
                // Tell the user to try again
                Alert alert = new Alert(javafx.scene.control.Alert.AlertType.ERROR);
                alert.setTitleText("Number Formatting Error");
                alert.setContentText(String.format("%s\n%s %s",
                        "Please input a whole number.", "Error:",
                        ex.getLocalizedMessage()));
                alert.showAndWait();

            }

        });
        cancel.setOnAction(e -> close());

        // Don't allow edits on the date/time controls
        datePicker.disableProperty().bind(editMode);
        ampmControl.disableProperty().bind(editMode);
        hourSpinner.disableProperty().bind(editMode);
        minuteSpinner.disableProperty().bind(editMode);
    }

    private void close() {

        clearControls();
        model.activeBPData().set(null);
        MobileApplication.getInstance().switchView(HOME_VIEW);
    }

    // Get hour from Spinner control and ampmControl.
    // Converts 1-12 hours with am/pm to a 24 (0-23) hour clock.
    // 12:15 am converts to 00:15, 4:25 pm converts to 16:25,
    // and 12:30 pm is 12:30
    private int getHourSpinner() {
        int hour = hourSpinner.getValue();
        if (hour == 12 && "am".equals(
                ampmControl.getSelectionModel().getSelectedItem())) {
            hour = 0;
        } else if (hour < 12 && "pm".equals(
                ampmControl.getSelectionModel().getSelectedItem())) {
            hour += 12;
        }
        return hour;
    }

    private void clearControls() {
//        datePicker.setValue(null);
//        hourFactory.setValue(1);
//        minuteFactory.setValue(0);
//        ampmControl.getSelectionModel().clearAndSelect(0);
        systolicField.setText("");
        diastolicField.setText("");
        pulseField.setText("");

    }

    // Set hour, minute, and ampm controls from the provided LocalDateTime.
    // Must convert the 24 (0-23) hour time to 1-12 with appropriate am/pm
    private void setTimeControls(LocalDateTime localDateTime) {
        int hour = localDateTime.getHour();
        minuteFactory.setValue(localDateTime.getMinute());
        ampmControl.getSelectionModel().select("am");

        if (hour >= 12) {
            ampmControl.getSelectionModel().select("pm");
            hour = (hour > 12) ? hour - 12 : hour;
        } else if (hour == 0) {
            hour = 12;
        }
        hourFactory.setValue(hour);
    }

}
