package com.asgteach.bpmonitor.views;

import com.gluonhq.charm.glisten.application.MobileApplication;
import com.gluonhq.charm.glisten.control.AppBar;
import com.gluonhq.charm.glisten.mvc.View;
import com.gluonhq.charm.glisten.visual.MaterialDesignIcon;
import com.asgteach.bpmonitor.BPMonitor;
import static com.asgteach.bpmonitor.BPMonitor.EDIT_VIEW;
import static com.asgteach.bpmonitor.BPMonitor.GRAPH_VIEW;
import static com.asgteach.bpmonitor.BPMonitor.STAT_VIEW;
import com.asgteach.bpmonitor.model.BPData;
import com.asgteach.bpmonitor.model.Model;
import com.asgteach.bpmonitor.model.MyData;
import com.asgteach.bpmonitor.service.Service;
import com.gluonhq.charm.glisten.animation.BounceInLeftTransition;
import com.gluonhq.charm.glisten.control.Alert;
import com.gluonhq.charm.glisten.control.Avatar;
import com.gluonhq.charm.glisten.control.CharmListView;
import com.gluonhq.charm.glisten.control.FloatingActionButton;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.util.Optional;
import javafx.collections.ListChangeListener;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Label;
import javafx.scene.control.Tooltip;
import javafx.scene.image.Image;
import javax.inject.Inject;

public class ReadingsPresenter {

    
    private Button chartButton;
    private Button homeButton;
    private Button statButton;

    @Inject
    private Model model;
    @Inject
    private Service service;

    @FXML
    private View readings;
    @FXML
    private CharmListView<BPData, Double> bpCharmList;

    public void initialize() {
        readings.setShowTransitionFactory(BounceInLeftTransition::new);
        readings.showingProperty().addListener((obs, oldValue, newValue) -> {
            if (newValue) {
                AppBar appBar = MobileApplication.getInstance().getAppBar();

                appBar.setTitleText("BP Readings");

                appBar.setNavIcon(new Avatar(17, new Image(BPMonitor.class.getResourceAsStream("/icon.png"))));

                appBar.getActionItems().add(homeButton = MaterialDesignIcon.HOME.button(e
                        -> System.out.println("Home View")));
                appBar.getActionItems().add(chartButton = MaterialDesignIcon.TIMELINE.button(e
                        -> MobileApplication.getInstance().switchView(GRAPH_VIEW)));
                appBar.getActionItems().add(statButton = MaterialDesignIcon.VIEW_WEEK.button(e
                        -> MobileApplication.getInstance().switchView(STAT_VIEW)));

                homeButton.setTooltip(new Tooltip("VIEW DATA"));
                chartButton.setTooltip(new Tooltip("VIEW LINE CHART"));
                statButton.setTooltip(new Tooltip("VIEW STATS"));
                homeButton.setDisable(true);

            }
        });

        bpCharmList.setHeadersFunction(ReadingsPresenter::getDateNum);
        // sort headers most recent first
        bpCharmList.setHeaderComparator((d1, d2)->d2.compareTo(d1));
        bpCharmList.setCellFactory(p -> new BPDataCell(this::edit, this::remove));
        bpCharmList.setHeaderCellFactory(p -> new BPHeaderCell());
        bpCharmList.setPlaceholder(new Label("There are no BP Readings.\nPress '+' to create BP Readings."));
        // sort list according to date/time--most recent first
        bpCharmList.setComparator((d1, d2) -> d2.getDate().compareTo(d1.getDate()));

        service.bpListProperty().addListener((ListChangeListener.Change<? extends BPData> c) -> {
            System.out.println("Service List Property listener: list size = " + c.getList().size());
            bpCharmList.setItems(service.bpListProperty().get());
            if (service.bpListProperty().isEmpty()) {
                MyData.fillBPData(service.bpListProperty().get());
            }
        });

        FloatingActionButton fb = new FloatingActionButton(MaterialDesignIcon.ADD.text,
                e -> {
                    // add new BP Data
                    model.activeBPData().set(null);
                    MobileApplication.getInstance().switchView(EDIT_VIEW);
                });
        fb.showOn(readings);

        service.retrieveReadings();

    }

    private void edit(BPData data) {
        System.out.println("Edit for reading " + data);
        model.activeBPData().set(data);
        MobileApplication.getInstance().switchView(BPMonitor.EDIT_VIEW);
    }

    private void remove(BPData data) {
        if (data == null) {
            System.out.println("Null data");
            return;
        }
        System.out.println("Remmove confirmation for reading " + data);
        Alert alert = new Alert(javafx.scene.control.Alert.AlertType.CONFIRMATION);
        alert.setTitleText("Confirm Delete");
        alert.setContentText("OK to Delete BP Reading?\n\n"
        + data.getSystolic()
        + " / " + data.getDiastolic()
        + " mmHg\n" + data.getPulse()
        + " bpm\n" + data.getDate().format(DateTimeFormatter.ofLocalizedDateTime(FormatStyle.SHORT)));

        Button yes = new Button("Yes-Delete");
        yes.setOnAction(e -> {
            alert.setResult(ButtonType.YES);
            alert.hide();
        });
        yes.setDefaultButton(true);

        Button no = new Button("No");
        no.setCancelButton(true);
        no.setOnAction(e -> {
            alert.setResult(ButtonType.NO);
            alert.hide();
        });
        alert.getButtons().setAll(yes, no);

        Optional result = alert.showAndWait();
        if (result.isPresent() && result.get().equals(ButtonType.YES)) {
            System.out.println("removing item: " + data);
            service.removeReading(data);
        }

    }

    private static Double getDateNum(BPData data) {
        LocalDate d = data.getDate().toLocalDate();
        int month = d.getMonthValue();
        double number = d.getYear() + (month * .01);
        return number;
    }


}
